import numpy as np
import model
import argparse
import matplotlib.pyplot as plt
import bin.functions as md
import bin.wrap as wrap
import bin.other as ot

import sys

ls = 299792458. # light speed


# Parse the command line options
parser = argparse.ArgumentParser(description='',
                                 formatter_class=argparse.RawTextHelpFormatter)
parser.add_argument("dataFile", metavar="dataFile.dat", nargs=1,
                    help="ASCII file containing Stokes spectra & errors.")
parser.add_argument("-fmin", dest="fmin", type=float, default=700.,
                    help="minimum frequency [MHz].")
parser.add_argument("-fmax", dest="fmax", type=float, default=1800.,
                    help="maximum frequency [MHz].")
parser.add_argument("-d", dest="d", type=int, default=0,
                    help="The number of delta functions.")
parser.add_argument("-g", dest="g", type=int, default=0,
                    help="The number of Gauss functions.")
parser.add_argument("-t", dest="t", type=int, default=0,
                    help="The number of tophat (boxcar) functions.")
parser.add_argument("-n", dest="n", type=int, default=10000,
                    help="The number of MCMC iteration. Default is 10000")
parser.add_argument("-b", dest="b", type=float, default=0.2,
                    help="The fraction of burn-in stage. Default is 0.2.")
# parser.add_argument("-p", dest="showPlots", action="store_true",
#                     help="show the plots [False].")
args = parser.parse_args()

if args.b<0 or args.b>1:
    print('Please set b as [0 <= b <= 1]')
    sys.exit()

Niter = args.n
Nb    = int(args.n*args.b)
Ns    = Niter-Nb

src_delta  = args.d
src_gauss  = args.g
src_boxcar = args.t

if src_delta==0 and src_gauss==0 and src_boxcar==0:
    print('Please set number of functions.')
    sys.exit()


fmin = args.fmin
fmax = args.fmax
l2min = md.FtoL2(fmax)
l2max = md.FtoL2(fmin)

f = np.loadtxt(args.dataFile[0])
f = np.delete(f,np.where(f<l2min)[0][np.where(np.where(f<l2min)[1]==0)],0)
f = np.delete(f,np.where(f>l2max)[0][np.where(np.where(f>l2max)[1]==0)],0)

print('')
print('start')
print('')
print('=== read data ===')
print('         Frequency: %d - %d [MHz]'%(args.fmin,args.fmax))
print('Squared wavelength: %5.3lf - %5.3lf [m^2]'%(l2min,l2max))
print('Number of channels: %d'%len(f))
print('Number of data    : %d (Stokes Q&U)'%(len(f)*2))
print('')
print('')
print('=== MCMC ===')
print('Iteration num: %d'%Niter)
print('  - burn-in  : %d'%Nb)
print('  - sampling : %d'%Ns)
print('Model        : d-%d g-%d t-%d'%(src_delta,src_gauss,src_boxcar))

seed = wrap.init_rand() # random number seed

lam2  = f[:,0]
freq  = md.L2toF(lam2)
q     = f[:,1]
u     = f[:,2]
q_err = f[:,3]
u_err = f[:,4]


Npar = 3*src_delta+4*src_gauss+4*src_boxcar

par  = np.empty([4,Npar],np.float64)
peri = np.empty([Npar],np.int32)
par, peri = model.set_par_init(src_delta,src_gauss,src_boxcar)


#
# burn-in with adjusting the stepwidth
#
if Nb > 0:
    val      = 0.3
    widthdec = 1.0/1.02
    widthinc = 1.02
    Nadjust  = 100
    NiterB   = 0
    chainB   = np.empty([0,Npar+1],dtype=np.float64)
    while NiterB <= Nb:
        chain, Nacpt, Nstep = wrap.qumc(
            lam2,q,u,q_err,u_err,Nadjust,src_delta,src_gauss,src_boxcar,par,peri)
        ratio = Nacpt*1.0/Nstep

        for i in range(Nadjust):
            chainB = np.append(chainB,np.array([chain[:,i]]),axis=0)

        for i in range(Npar):
            if ratio[i] < val:
                par[1,i] = par[1,i]*widthdec
            else:
                par[1,i] = par[1,i]*widthinc

        par[0,:] = chain[0:Npar,Nadjust-1]

        NiterB = NiterB + Nadjust

    par[0,:] = chain[0:Npar,-1]


#
# sampling
#
chain, Nacpt, Nstep = wrap.qumc(
   lam2,q,u,q_err,u_err,Ns,src_delta,src_gauss,src_boxcar,par,peri)


#
# check bestfit parameters
#
bfloc   = np.argmin(chain[Npar])
bfpar   = chain[0:Npar,bfloc]
bfchisq = chain[Npar,bfloc]


#
# calculate chisq, AIC, BIC
#
print('')
print('')
print('=== results ===')
print('')
print('*best fit parameters*')
ot.show_bfpar(src_delta,src_gauss,src_boxcar,bfpar)
aic    = bfchisq + 2.*Npar
bic    = bfchisq + Npar*np.log(2.*len(lam2))
chisqr = bfchisq/(2.*len(lam2)-Npar)
print('')
print('*Chisq. value & information criteria*')
print('')
print('        chisq: '+str(bfchisq))
print('reduced chisq: '+str(chisqr))
print('          AIC: '+str(aic))
print('          BIC: '+str(bic))


#
# plots
#
print('')
print('')
print('=== plotting... ===')
ot.plot_spec(freq,lam2,q,u,q_err,u_err,src_delta,src_gauss,src_boxcar,bfpar)
ot.plot_1d(src_delta,src_gauss,src_boxcar,chain)
plt.show()
